; **************************************************************************
; DIALOGS.SC - a generic dialog box library to create DIALOGS.LIB
;
; AUTHOR           DATE            VER  DETAILS
;
; John O'Connell   10th Nov 1993   1.0  Initial implementation
;
;
; Feel free to use this script in your applications, all I ask is that you
; don't remove these comments.  If you change this code and distribute those
; changes to others then please keep the above revision comments updated.
;
; I think the PROC names are self-explanatory but I'll explain the common
; parameters passed to the MsgXXXXXX PROCs ...
;
; Title    - the title displayed in the dialog box .
; MsgText  - the message text displayed in the dialog box.  By embedding
;            \n characters within this string you can break the text into
;            individual lines of text.  The dialog box will auto-size.
;
; OKLbl    - the label displayed within the OK button  *
; YesLbl   - the label displayed within the Yes button **
; NoLbl    - the label displayed within the No button  **
; CancLbl  - the label displayed within the Cancel button ***
;
; Passing these parameters as blank strings will default to the appropriate
; button labels (i.e. ~O~k, ~Y~es, ~N~o, ~C~ancel)
;
; Centred  - the logical value specifying that text is to be centred
;
; *     - used by MsgOK
; **    - used by MsgYesNo, MsgNoYes and MsgYesNoCancel
; ***   - used by MsgYesNoCancel
;
; NOTE that all the MsgXXXX PROCs except for MsgOK return a value.  I'll
; leave you to work out what's returned from the code ...
;
; Any feedback to me via Compuserve Mail 73064,74
;
; John O'Connell
;
; **************************************************************************
CREATELIB "Dialogs"
; **************************************************************************
; Flush()
;
; This flushes the keyboard of any pending keystrokes.  Called by MsgXXXXX
; **************************************************************************
PROC Flush()
PRIVATE Ch
   WHILE CHARWAITING()
      Ch = GETCHAR()
   ENDWHILE
ENDPROC

WRITELIB "Dialogs" Flush
RELEASE PROCS Flush

; **************************************************************************
; MsgDialogProc()
;
; This PROC is a general dialog procedure for all the other dialog PROCs
; defined below.  It is called by the OPEN trigger for a SHOWDIALOG and
; displays each text string defined in the DYNARRAY TxtArray.
;
; If the first character in a line of text is found to be ASCII 128 then
; that line of text will blink.
; ***************************************************************************
PROC MsgDialogProc(TriggerVal, TagVal, EventVal, ElementVal)
PRIVATE DlgCanvas, TxtLen, CurrRow, FmtSpec

   ; if the CENTRED parameter passed to the MsgXXXXX procedure is
   ; True then centre each line of text within the width of the
   ; longest line of text in the privately declared DYNARRAY TxtArray
   
   FmtSpec = IIF(Centred, "ac,w", "al,w") + STRVAL(MaxLen)
   TxtLen  = DYNARRAYSIZE(TxtArray)

   ; set the canavs to the current dialog - the calling dialog
   WINDOW HANDLE DIALOG TO DlgCanvas
   SETCANVAS dlgCanvas
   
   CurrRow = TxtLen
   
   FOREACH TxtPtr IN TxtArray
     ; if the first character in the line of text is ASCII 128
     ; then set the text colour to have the blinking attribute ...
     IF MATCH(TxtArray[TxtPtr], "\128..", TxtArray[TxtPtr]) THEN
        STYLE ATTRIBUTE SYSCOLOR(1036) + 128
     ENDIF
     
     @ CurrRow, Margin ?? FORMAT(FmtSpec, TxtArray[TxtPtr])
     STYLE ATTRIBUTE SYSCOLOR(1036)
     CurrRow = CurrRow - 1
   ENDFOREACH

   RETURN True

ENDPROC

WRITELIB "Dialogs" MsgDialogProc
RELEASE PROCS MsgDialogProc

; **************************************************************************
PROC MsgYesNoCancel(Title, MsgText, YesLbl, NoLbl, CancLbl, Centred)
PRIVATE SizeX, PosCol, PosRow, PosButton1, PosButton2, ButtonVal,
        PosButtonRow, DlgHeight, PosButton3, TxtArray, TxtPtr,
        Margin, MaxLen, ButtWidth

DYNARRAY TxtArray[]

   ; calculate max button width
   ButtWidth = MAX(10, MAX( LEN(YesLbl),
                       MAX( LEN(NoLbl), LEN(CancLbl) ) ) + 4 )

   ; separate the message text into individual lines if necessary ...
   TxtPtr = 1
   WHILE MATCH(MsgText, "..\n..", TxtArray[TxtPtr], MsgText)
      TxtPtr = TxtPtr + 1
   ENDWHILE
   
   ; account for the "trailing" text ...
   IF LEN(MsgText) > 0 THEN
      TxtArray[TxtPtr] = MsgText
   ELSE
      TxtPtr = TxtPtr - 1
   ENDIF
   
   SizeX = MAX(LEN(Title) + 8, 45)

   DlgHeight    = TxtPtr + 6
   PosButtonRow = TxtPtr + 2

   IF LEN(YesLbl) = 0 THEN
      YesLbl = "~Y~es"
   ENDIF

   IF LEN(NoLbl) = 0 THEN
      NoLbl = "~N~o"
   ENDIF

   IF LEN(CancLbl) = 0 THEN
      CancLbl = "~C~ancel"
   ENDIF

   Flush()

   ; work out the dialog box dimensions ...
   MaxLen = 0
   FOREACH TxtPtr IN TxtArray
      SizeX  = MAX(SizeX, LEN(TxtArray[TxtPtr]))
      MaxLen = MAX(MaxLen, LEN(TxtArray[TxtPtr]))
   ENDFOREACH

   SizeX = MAX(SizeX + 4, ButtWidth * 3 + 8)

   Margin = INT( (SizeX - MaxLen) / 2)
   PosRow = INT( (23 - DlgHeight) / 2)
   PosCol = INT( (78 - SizeX) / 2 )

   PosButton1 =  INT( SizeX / 6 ) - INT(ButtWidth/2)
   PosButton2 = (INT( SizeX / 6 ) * 3) - INT(ButtWidth/2)
   PosButton3 = (INT( SizeX / 6 ) * 5) - INT(ButtWidth/2)

   SHOWDIALOG Title
      PROC "MsgDialogProc" TRIGGER "OPEN"

      @ PosRow, PosCol HEIGHT DlgHeight WIDTH SizeX + 2

      PUSHBUTTON @ PosButtonRow, PosButton1 WIDTH ButtWidth
        YesLbl
        OK
        DEFAULT
        VALUE YesLbl
        TAG "tgYes"
      TO ButtonVal

      PUSHBUTTON @ PosButtonRow, PosButton2 WIDTH ButtWidth
        NoLbl
        OK
        VALUE NoLbl
        TAG "tgNo"
      TO ButtonVal

      PUSHBUTTON @ PosButtonRow, PosButton3 WIDTH ButtWidth
        CancLbl
        CANCEL
        VALUE CancLbl
        TAG "tgCancel"
      TO ButtonVal
      
   ENDDIALOG
   
   RETURN IIF(Retval, ButtonVal, CancLbl)

ENDPROC

WRITELIB "Dialogs" MsgYesNoCancel
RELEASE PROCS MsgYesNoCancel

; **************************************************************************
PROC MsgNoYes(Title, MsgText, NoLbl, YesLbl, Centred)
PRIVATE SizeX, PosRow, PosCol, PosButton1, PosButton2, ButtonVal,
        PosButtonRow, DlgHeight, TxtArray, TxtPtr, Margin, MaxLen, ButtWidth

DYNARRAY TxtArray[]

   ; calculate max button width
   ButtWidth = MAX(10, MAX( LEN(YesLbl), LEN(NoLbl) ) + 4 )

   ; separate the message text into individual lines if necessary ...
   TxtPtr = 1
   WHILE MATCH(MsgText, "..\n..", TxtArray[TxtPtr], MsgText)
      TxtPtr = TxtPtr + 1
   ENDWHILE
   
   ; account for the "trailing" text ...
   IF LEN(MsgText) > 0 THEN
      TxtArray[TxtPtr] = MsgText
   ELSE
      TxtPtr = TxtPtr - 1
   ENDIF
   
   SizeX = MAX(LEN(Title) + 8, 30)
   
   DlgHeight = TxtPtr + 6
   PosButtonRow = TxtPtr + 2

   IF LEN(YesLbl) = 0 THEN
      YesLbl = "~Y~es"
   ENDIF

   IF LEN(NoLbl) = 0 THEN
      NoLbl = "~N~o"
   ENDIF

   Flush()

   ; work out the dialog box dimensions ...
   MaxLen = 0
   FOREACH TxtPtr IN TxtArray
      SizeX  = MAX(SizeX, LEN(TxtArray[TxtPtr]))
      MaxLen = MAX(MaxLen, LEN(TxtArray[TxtPtr]))
   ENDFOREACH

   SizeX = MAX(SizeX + 4, ButtWidth * 2 + 6)

   Margin = INT( (SizeX - MaxLen) / 2)
   PosRow = INT( (23 - DlgHeight) / 2)
   PosCol = INT( (78 - SizeX) / 2 )

   PosButton1  =  INT( SizeX / 4 )      - INT(ButtWidth/2)
   PosButton2  = (INT( SizeX / 4 ) * 3) - INT(ButtWidth/2)

   SHOWDIALOG Title
      PROC "MsgDialogProc" TRIGGER "OPEN"

      @ PosRow, PosCol HEIGHT DlgHeight WIDTH SizeX + 2

      PUSHBUTTON @ PosButtonRow, PosButton1 WIDTH ButtWidth
        NoLbl
        CANCEL
        DEFAULT
        VALUE "No"
        TAG "tgNo"
      TO ButtonVal

      PUSHBUTTON @ PosButtonRow, PosButton2 WIDTH ButtWidth
        YesLbl
        OK
        VALUE "Yes"
        TAG "tgYes"
      TO ButtonVal
      
   ENDDIALOG
   
   RETURN Retval

ENDPROC

WRITELIB "Dialogs" MsgNoYes
RELEASE PROCS MsgNoYes

; **************************************************************************
PROC MsgYesNo(Title, MsgText, YesLbl, NoLbl, Centred)
PRIVATE SizeX, PosRow, PosCol, PosButton1, PosButton2, ButtonVal,
        PosButtonRow, DlgHeight, TxtArray, TxtPtr, Margin, MaxLen, ButtWidth

DYNARRAY TxtArray[]

   ; calculate max button width
   ButtWidth = MAX(10, MAX( LEN(YesLbl), LEN(NoLbl) ) + 4 )

   ; separate the message text into individual lines if necessary ...
   TxtPtr = 1
   WHILE MATCH(MsgText, "..\n..", TxtArray[TxtPtr], MsgText)
      TxtPtr = TxtPtr + 1
   ENDWHILE
   
   ; account for the "trailing" text ...
   IF LEN(MsgText) > 0 THEN
      TxtArray[TxtPtr] = MsgText
   ELSE
      TxtPtr = TxtPtr - 1
   ENDIF
   
   SizeX = MAX(LEN(Title) + 8, 30)
   
   DlgHeight = TxtPtr + 6
   PosButtonRow = TxtPtr + 2

   IF LEN(YesLbl) = 0 THEN
      YesLbl = "~Y~es"
   ENDIF

   IF LEN(NoLbl) = 0 THEN
      NoLbl = "~N~o"
   ENDIF

   Flush()

   ; work out the dialog box dimensions ...
   MaxLen = 0
   FOREACH TxtPtr IN TxtArray
      SizeX  = MAX(SizeX, LEN(TxtArray[TxtPtr]))
      MaxLen = MAX(MaxLen, LEN(TxtArray[TxtPtr]))
   ENDFOREACH

   SizeX = MAX(SizeX + 4, ButtWidth * 2 + 6)

   Margin = INT( (SizeX - MaxLen) / 2)
   PosRow = INT( (23 - DlgHeight) / 2)
   PosCol = INT( (78 - SizeX) / 2 )

   PosButton1  =  INT( SizeX / 4 )      - INT(ButtWidth/2)
   PosButton2  = (INT( SizeX / 4 ) * 3) - INT(ButtWidth/2)

   SHOWDIALOG Title
      PROC "MsgDialogProc" TRIGGER "OPEN"

      @ PosRow, PosCol HEIGHT DlgHeight WIDTH SizeX + 2

      PUSHBUTTON @ PosButtonRow, PosButton1 WIDTH ButtWidth
        YesLbl
        OK
        DEFAULT
        VALUE "Yes"
        TAG "tgYes"
      TO ButtonVal

      PUSHBUTTON @ PosButtonRow, PosButton2 WIDTH ButtWidth
        NoLbl
        CANCEL
        VALUE "No"
        TAG "tgNo"
      TO ButtonVal
      
   ENDDIALOG
   
   RETURN Retval

ENDPROC

WRITELIB "Dialogs" MsgYesNo
RELEASE PROCS MsgYesNo

; **************************************************************************
PROC MsgOK(Title, MsgText, OKLbl, Centred)
PRIVATE SizeX, PosCol, PosRow, PosButton, ButtonVal, DlgHeight, ButtWidth,
        PosButtonRow, TxtArray, TxtPtr, Margin, MaxLen

DYNARRAY TxtArray[]

   ; calculate max button width
   ButtWidth = MAX(10, LEN(OKLbl) + 4)

   ; separate the message text into individual lines if necessary ...
   TxtPtr = 1
   WHILE MATCH(MsgText, "..\n..", TxtArray[TxtPtr], MsgText)
      TxtPtr = TxtPtr + 1
   ENDWHILE
   
   ; account for the "trailing" text ...
   IF LEN(MsgText) > 0 THEN
      TxtArray[TxtPtr] = MsgText
   ELSE
      TxtPtr = TxtPtr - 1
   ENDIF
   
   SizeX = MAX(LEN(Title) + 8, 10)
   
   IF LEN(OKLbl) = 0 THEN
      OKLbl = "~O~K"
   ENDIF

   DlgHeight    = TxtPtr + 6
   PosButtonRow = TxtPtr + 2
   
   Flush()

   ; work out the dialog box dimensions ...
   MaxLen = 0
   FOREACH TxtPtr IN TxtArray
      SizeX  = MAX(SizeX, LEN(TxtArray[TxtPtr]))
      MaxLen = MAX(MaxLen, LEN(TxtArray[TxtPtr]))
   ENDFOREACH

   SizeX = MAX(SizeX + 4, ButtWidth + 4)

   Margin = INT( (SizeX - MaxLen) / 2)
   PosRow = INT( (23 - DlgHeight) / 2)
   PosCol = INT( (78 - SizeX) / 2 )

   PosButton = INT( SizeX / 2 ) - INT(ButtWidth/2)   ; centre pushbutton

   SHOWDIALOG Title
      PROC "MsgDialogProc" TRIGGER "OPEN"

      @ 7, PosCol HEIGHT DlgHeight WIDTH SizeX + 2
      
      PUSHBUTTON @ PosButtonRow, PosButton WIDTH ButtWidth
       OKLbl
       OK
       VALUE "OK"
       TAG "tgOK"
      TO ButtonVal
      
   ENDDIALOG

   ; no return value as none required in this case

ENDPROC

WRITELIB "Dialogs" MsgOK
RELEASE PROCS MsgOK
